﻿using System;
using System.Collections.Generic;

namespace Graphics
{
    public class Layer
    {
        public char[,] image;
        public char[,] Bcolor;
        public char[,] Fcolor;
        public List<Sprite> items;
        public List<pObj> printQueue;
        private int h = Console.BufferHeight;
        private int w = Console.BufferWidth;

        public class pObj
        {
            public int x, y;
            public string s ="";
            public bool color = false;
            public ConsoleColor f, b;
        }

        //Backup
        private char[,] _image;
        private char[,] _Bcolor;
        private char[,] _Fcolor;

        public Layer()
        {
            image = new char[w, h];
            Bcolor = new char[w, h];
            Fcolor = new char[w, h];
            items = new List<Sprite>();
        }

        public void add(Sprite s)
        {
            items.Add(s);
        }

        public void add(ICollection<Sprite> collection)
        {
            foreach (Object o in collection)
            {
                Sprite s = (Sprite) o;
                add(s);
            }
        }

        public void remove(Sprite s)
        {
            items.Remove(s);
        }

        public void remove(ICollection<Sprite> collection)
        {
            foreach (Object o in collection)
            {
                Sprite s = (Sprite)o;
                remove(s);
            }
        }

        public void readyPrint()
        {
            for (int c = 0; c < items.Count; c++)
            {
                items[c].onPrint();
                if (items[c].disabled)
                    remove(items[c--]);
            }

            createArrays();
            createQueue();
        }

        public void print()
        {
            if (Graphics.Defaults.defaultBackground != Console.BackgroundColor)
                Console.BackgroundColor = Graphics.Defaults.defaultBackground;
            if (Graphics.Defaults.defaultForeground != Console.ForegroundColor) 
                Console.ForegroundColor = Graphics.Defaults.defaultForeground;

            foreach (pObj p in printQueue)
            {
                Console.SetCursorPosition(p.x, p.y);
                if (p.color)
                {
                    Console.BackgroundColor = p.b;
                    Console.ForegroundColor = p.f;
                }

                Console.Write(p.s);

                if (p.color)
                    Console.ResetColor();
            }
        }

        //PRIVATE METHOD
        private void createArrays()
        {
            //Backup arrays
            _image = image;
            _Bcolor = Bcolor;
            _Fcolor = Fcolor;

            image = new char[w, h];
            Bcolor = new char[w, h];
            Fcolor = new char[w, h];

            foreach (Sprite s in items)
            {
                if (s.image != null)
                {
                    for (int y = 0; y < s.image.Length; y++)
                    {
                        if (s.image[y] != null)
                        {
                            for (int x = 0; x < s.image[y].Length; x++)
                            {
                                char c = s.image[y].ToCharArray(x, 1)[0];

                                //If this is not a transparent spot AND is on the screen
                                if ((!s.hasTransparency || c != s.transparencyKey) &&
                                    s.xP + x < Console.BufferWidth && s.xP + x > -1 &&
                                    s.yP + y < Console.BufferHeight && s.yP + y > -1)
                                {
                                    //Image
                                    image[s.xP + x, s.yP + y] = c;

                                    //If the image has color
                                    if (s.hasColor)
                                    {
                                        //Bcolor
                                        if (s.bcolor != null)
                                            Bcolor[s.xP + x, s.yP + y] = s.bcolor[y].ToCharArray(x, 1)[0];
                                        else
                                            Bcolor[s.xP + x, s.yP + y] = Graphics.Defaults.getChar(s.backgroundColor);

                                        //Fcolor
                                        if (s.fcolor != null)
                                            Fcolor[s.xP + x, s.yP + y] = s.fcolor[y].ToCharArray(x, 1)[0];
                                        else
                                            Fcolor[s.xP + x, s.yP + y] = Graphics.Defaults.getChar(s.foregroundColor);
                                    }
                                }
                            }//end of x
                        }
                    }//end of y
                }
            }//end of foreach Sprite
        }

        private void createQueue()
        {
            printQueue = new List<pObj>();
            pObj p = new pObj();

            for (int y = 0; y < h; y++)
            {
                for (int x = 0; x < w; x++)
                {
                    if (image[x, y] != _image[x, y] || Fcolor[x, y] != _Fcolor[x, y] || Bcolor[x, y] != _Bcolor[x, y])
                    {
                        if (p.s.Length == 0)
                        {
                            p.x = x;
                            p.y = y;
                        }

                        p.s += image[x, y];

                        //Color
                        if (Fcolor[x, y] != '\0' || Bcolor[x, y] != '\0')
                        {
                            if (p.s.Length > 1 && !p.color) {
                                p.s = p.s.Substring(0, p.s.Length - 1);
                                printQueue.Add(p);
                                p = new pObj();
                                p.x = x;
                                p.y = y;
                                p.s += image[x, y];
                                p.color = true;
                                p.b = Graphics.Defaults.getColor(Bcolor[x, y]);
                                p.f = Graphics.Defaults.getColor(Fcolor[x, y]);
                            }
                            else {
                                p.color = true;
                                p.b = Graphics.Defaults.getColor(Bcolor[x, y]);
                                p.f = Graphics.Defaults.getColor(Fcolor[x, y]);
                            }
                        }
                        else if (p.color)
                        {
                            p.s = p.s.Substring(0, p.s.Length - 1);
                            printQueue.Add(p);
                            p = new pObj();
                            p.x = x;
                            p.y = y;
                            p.s += image[x, y];
                        }
                    }
                    else if (p.s.Length != 0)
                    {
                        printQueue.Add(p);
                        p = new pObj();
                    }
                }//end of x
            }//end of y
        }
    }
}