﻿using System;
using System.Collections.Generic;

namespace Graphics
{
    /// <summary>
    /// Represents an animation implementation of a Sprite
    /// </summary>
    public class Sub : Sprite
    {
        public int counter = 0;
        public int moveC = 0;
        public List<string[]> frames = new List<string[]>();
        public List<string[]> solids = new List<string[]>();
        public List<int> xPs = new List<int>();
        public List<int> yPs = new List<int>();

        public bool submerged = false;
        public bool transition = false;
        public string flip = "_L";
        private static Random r = new Random();

        public int subtimer;

        public Sub(int x, int y, bool _submerged)
        {
            xP = x;
            yP = y;
            submerged = _submerged;
            setSubPath();
            move(null);
            subtimer = r.Next(5,45);

            if (submerged)
                setpath(@"Graphics\Library\Sub\submerged" + flip);
            else
                setpath(@"Graphics\Library\Sub\surfaced" + flip);
        }

        private void setpath(string path)
        {
            string[] entireFile = Graphics.Defaults.fileToStringArray(path);
            int h = 0;
            _path = path.Substring(0, path.Length - 2);

            frames = new List<string[]>();
            counter = 0;

            //Set params
            if (entireFile[0].Contains("KEY"))
            {
                hasTransparency = true;
                transparencyKey = entireFile[0].ToCharArray()[entireFile[0].IndexOf("KEY") + 4];
            }
            if (entireFile[0].Contains("HEIGHT"))
            {
                h = Int32.Parse(entireFile[0].Substring(entireFile[0].IndexOf("HEIGHT") + 7, 1));
            }

            //Set Image
            string[] allFrames = new string[entireFile.Length - 1];
            for (int i = 0; i < allFrames.Length; i++)
                allFrames[i] = entireFile[i + 1];
            this.setFrames(allFrames, h, frames);

            //Get collision data
            string[] entireFile2 = Graphics.Defaults.fileToStringArray(path + ".col");
            if (entireFile2 != null)
            {
                if (entireFile2[0].Contains("HEIGHT"))
                {
                    h = Int32.Parse(entireFile2[0].Substring(entireFile2[0].IndexOf("HEIGHT") + 7, 1));
                    //Set Image
                    string[] allFrames2 = new string[entireFile2.Length - 1];
                    for (int i = 0; i < allFrames2.Length; i++)
                        allFrames2[i] = entireFile2[i + 1];
                    this.setFrames(allFrames2, h, solids);
                }
                else
                {
                    solid = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        solid[i] = entireFile2[i];
                }
            }
        }

        private void setFrames(string[] f, int h, List<string[]> dest)
        {
            if (h == 0)
                h = f.Length;
            string[] frame = new string[h];
            for (int i = 0; i < f.Length; )
            {
                frame[i % h] = f[i];
                i++;

                if (i % h == 0)
                {
                    dest.Add(frame);
                    frame = new string[h];
                }
            }

            if (dest.Equals(frames))
                image = dest[0];
        }

        public override void onPrint()
        {
            _path = @"Graphics\Library\Sub\submerge"; //insures the proper sinking animation

            //Increment counter
            if (counter == frames.Count)
            {
                transition = false;
                return;
            }

            //Adjust image
            if (frames.Count > 0)
            {
                image = frames[counter];
                counter++;
            }

            //Temp fix for collision data acting weird
            if (image[3].Contains(")"))
            {
                string[] s = { "????|?????????", "??-++-????????", "?XXXXXXXXXXXX?", "XXXXXXXXXXXXXX" };
                solid = s;
            }
            else if (image[3].Contains("_"))
            {
                string[] s = { "????|?????????", "????|?????????", "??-++-????????", "?XXXXXXXXXXXX?" };
                solid = s;
            }
            else
            {
                string[] s = { "????|?????????", "????|?????????", "????|?????????", "????|?????????", };
                solid = s;
            }

        }

        /// <summary>
        /// Sets a random sub path that loops
        /// </summary>
        private void setSubPath()
        {
            Random r = new Random();

            int x, y, a, b;
            x = xP;
            y = yP;

            subtimer = r.Next(15, 50);

            for (int i = 0; i < 20; i++)
            {
                do
                {
                    a = r.Next(0, 105);
                    b = r.Next(Math.Max(0, y - 10), Math.Min(55, y + 10));
                } while (Math.Abs(Math.Sqrt((a - x) * (a - x) + (b - y) * (b - y))) < 20);

                setPath(x,y, a, b);

                x = a;
                y = b;
            }

            setPath(x, y, xP, yP);
        }

        /// <summary>
        /// This method is called when a sub should move (not every frame)
        /// </summary>
        /// <param name="col"></param>
        public void move(List<Sprite> col)
        {
            bool changed = flipped;

            xP = xPs[moveC];
            yP = yPs[moveC];
            moveC = (moveC + 1) % xPs.Count;

            flipped = xP - xPs[moveC] > 0;
            if (flipped)
                flip = "_L";
            else
                flip = "_R";

            //Change orientation if necessary
            if (changed != flipped && counter == frames.Count)
            {
                if (submerged)
                    setpath(@"Graphics\Library\Sub\submerged" + flip);
                else
                    setpath(@"Graphics\Library\Sub\surfaced" + flip);
            }

            subtimer = Math.Max(0, subtimer-1);

            if (subtimer == 0)
            {
                if (submerged)
                {
                    subtimer = r.Next(12, 30);
                    setSubSurface(); 
                }
                else
                {
                    subtimer = r.Next(10, 15);
                    setSubSubmerge();
                }
            }
        }

        public void setSubSubmerge()
        {
            transition = true;
            submerged = true;

            setpath(@"Graphics\Library\Sub\submerge" + flip);
        }

        public void setSubSurface()
        {
            transition = true;
            submerged = false;

            setpath(@"Graphics\Library\Sub\surface" + flip);
        }

        private void setPath(int x, int y, int a, int b)
        {
            for (int i = 0; i < Math.Abs(x - a); i++)
            {
                if (x>a)
                    xPs.Add(x-i);
                else
                    xPs.Add(x+i);

                double d = (1.0*(y-b))*(1.0 * i) / (1.0*Math.Abs(x - a));
                d = y - d;
                yPs.Add((int) d);
            }
        }
    }
}
