﻿using System;
using System.Collections.Generic;

namespace Graphics
{
    /// <summary>
    /// Represents an animation implementation of a Sprite
    /// </summary>
    public partial class Animation : Sprite
    {
        //public ConsoleColor foregroundColor = Graphics.Defaults.defaultForeground;
        //public ConsoleColor backgroundColor = Graphics.Defaults.defaultBackground;
        //public char transparencyKey = Graphics.Defaults.defaultTransparencyKey;

        //public bool hasTransparency = false;
        //public bool disabled = false;
        //public bool hasColor = false;

        //public string[] image;
        //public string[] fcolor;
        //public string[] bcolor;
        //public string[] solid;

        //public int xP, yP;

        public int counter = 0;
        public List<string[]> frames = new List<string[]>();
        public List<string[]> solids = new List<string[]>();
        public List<string[]> fcolors = new List<string[]>();
        public List<int> xPs = new List<int>();
        public List<int> yPs = new List<int>();
        public bool repeat = true;
        public int inc = -1;

        public Animation()
        { 
        }

        public Animation(string path, int x, int y)
        {
            xP = x;
            yP = y;
            setpath(path);
        }

        public Animation(string path)
        {
            setpath(path);
        }

        private void setpath(string path)
        {
            string[] entireFile = Graphics.Defaults.fileToStringArray(path);
            int h=0;
            _path = path.Substring(0, path.Length - 2);

            //Set params
            if (entireFile[0].Contains("KEY"))
            {
                hasTransparency = true;
                transparencyKey = entireFile[0].ToCharArray()[entireFile[0].IndexOf("KEY") + 4];
            }
            if (entireFile[0].Contains("BCOLOR"))
            {
                hasColor = true;
                backgroundColor = Graphics.Defaults.getColor(entireFile[0].ToCharArray()[entireFile[0].IndexOf("BCOLOR") + 7]);
            }
            if (entireFile[0].Contains("FCOLOR"))
            {
                hasColor = true;
                foregroundColor = Graphics.Defaults.getColor(entireFile[0].ToCharArray()[entireFile[0].IndexOf("FCOLOR") + 7]);
            }
            if (entireFile[0].Contains("REPEAT"))
            {
                repeat = true;
            }
            if (entireFile[0].Contains("ONCE"))
            {
                repeat = false;
            }
            if (entireFile[0].Contains("HEIGHT"))
            {
                h = Int32.Parse(entireFile[0].Substring(entireFile[0].IndexOf("HEIGHT")+7,1));
            }

            //Set Image
            string[] allFrames = new string[entireFile.Length - 1];
            for (int i = 0; i < allFrames.Length; i++)
                allFrames[i] = entireFile[i + 1];
            this.setFrames(allFrames, h, frames);

            //Get collision data
            string[] entireFile2 = Graphics.Defaults.fileToStringArray(path+".col");
            if (entireFile2 != null)
            {
                if (entireFile2[0].Contains("HEIGHT"))
                {
                    h = Int32.Parse(entireFile2[0].Substring(entireFile2[0].IndexOf("HEIGHT") + 7, 1));
                    //Set Image
                    string[] allFrames2 = new string[entireFile2.Length - 1];
                    for (int i = 0; i < allFrames2.Length; i++)
                        allFrames2[i] = entireFile2[i + 1];
                    this.setFrames(allFrames2, h, solids);
                }
                else
                {
                    solid = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        solid[i] = entireFile2[i];
                }
            }

            //Get bcolor data
            entireFile = Graphics.Defaults.fileToStringArray(path + ".b");
            if (entireFile != null)
            {
                hasColor = true;
                bcolor = new string[image.Length];
                for (int i = 0; i < image.Length; i++)
                    bcolor[i] = entireFile[i];
            }

            //Get fcolor data
            string[] entireFile1 = Graphics.Defaults.fileToStringArray(path + ".f");
            if (entireFile1 != null)
            {
                hasColor = true;

                if (entireFile1[0].Contains("HEIGHT"))
                {
                    h = Int32.Parse(entireFile1[0].Substring(entireFile1[0].IndexOf("HEIGHT") + 7, 1));
                    //Set Image
                    string[] allFrames1 = new string[entireFile1.Length - 1];
                    for (int i = 0; i < allFrames1.Length; i++)
                        allFrames1[i] = entireFile1[i + 1];
                    this.setFrames(allFrames1, h, fcolors);
                }
                else
                {
                    fcolor = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        fcolor[i] = entireFile1[i];
                }
            }
        }

        private void setFrames(string[] f, int h, List<string[]> dest)
        {
            if (h == 0)
                h = f.Length;
            string[] frame = new string[h];
            for (int i = 0; i < f.Length; )
            {
                frame[i % h] = f[i];
                i++;

                if (i % h == 0)
                {
                    dest.Add(frame);
                    frame = new string[h];
                }
            }

            if (dest.Equals(frames))
                image = dest[0];
            if (dest.Equals(fcolors))
                fcolor = dest[0];
        }

        public override void onPrint()
        {
            if (inc == -1)
                inc = (Math.Max(1, fcolors.Count) * Math.Max(1, solids.Count) * Math.Max(1, frames.Count) * Math.Max(1, xPs.Count) * Math.Max(1, yPs.Count));

            //Increment counter
            if (repeat)
                counter = ++counter % inc;
            else if (++counter == Math.Max(Math.Max(frames.Count, xPs.Count), yPs.Count))
            {
                disabled = true;
                return;
            }

            //Adjust image
            if (frames.Count > 0)
                image = frames[counter % frames.Count];

            //Adjust path
            if (xPs.Count > 0)
                xP = xPs[counter % xPs.Count];
            if (yPs.Count > 0)
                yP = yPs[counter % yPs.Count];

            //Adjust solid
            if (solids.Count > 0)
                solid = solids[counter % solids.Count];

            if (fcolors.Count > 0)
                fcolor = fcolors[counter % fcolors.Count];
        }
    }
}
