﻿using System;
using System.Collections.Generic;
using System.Threading;

namespace Sound
{
    /// <summary>
    /// A class that controls audio playback.  Switch between sound card and PC speaker.
    /// 
    /// Valid sndcrd formats: .ogg, .mp3, .wav
    /// Valid pcskr formats: .txt ()
    /// </summary>
    public class Player
    {
        public static bool mute = true;
        public static bool useSoundCard = false;
        public static bool availableSoundCard = false;

        public static Queue<string> Music_FileNames = new Queue<string>(); //do not include directory or extension
        public static Queue<string> FX_FileNames = new Queue<string>();    //do not include directory or extension
        private static Thread player = new Thread(new ThreadStart(runPlayer));

        public static Queue<int> pcspkr_Music, pcspkr_FX;       //Music will be looped

        /// <summary>
        /// Run once at the start of the application.
        /// </summary>
        public static void initialize()
        {
            availableSoundCard = false;
            
            pcspkr_FX = new Queue<int>();
            pcspkr_Music = new Queue<int>();
            player.Start();
        }

        /// <summary>
        /// Player loop
        /// </summary>
        private static void runPlayer()
        {
            string currentSong="";

            do
            {
                if (useSoundCard && availableSoundCard)
                {//Using sound card
                    
                }
                else
                {//Using PC speaker
                    //Play next note
                    if (pcspkr_FX.Count > 0)
                        PCSpkr_Play_Note(pcspkr_FX.Dequeue());
                    else if (pcspkr_Music.Count > 0)
                    {
                        PCSpkr_Play_Note(pcspkr_Music.Peek());
                        pcspkr_Music.Enqueue(pcspkr_Music.Dequeue());
                    }
                }

                if (Music_FileNames.Count > 0)
                {//Switch songs
                    string q = Music_FileNames.Dequeue();
                    if (q != currentSong)
                    {//If the song isn't already playing, switch song
                        currentSong = q;
                        load(@"Sound\Songs\" + q, true);

                        if (useSoundCard)
                            fade();

                        if (System.IO.File.Exists(@"Sound\Songs\" + q + ".ogg"))
                            q += ".ogg";
                        else if (System.IO.File.Exists(@"Sound\Songs\" + q + ".mp3"))
                            q += ".mp3";
                        else if (System.IO.File.Exists(@"Sound\Songs\" + q + ".wav"))
                            q += ".wav";
                        else
                            q = "Silence.wav";
                        
                    }
                }
                if (FX_FileNames.Count > 0)
                {//Add effect to engine 
                    string f = FX_FileNames.Dequeue();
                    load(@"Sound\FX\" + f, false);

                    if (System.IO.File.Exists(@"Sound\FX\" + f + ".ogg"))
                        f += ".ogg";
                    else if (System.IO.File.Exists(@"Sound\FX\" + f + ".mp3"))
                        f += ".mp3";
                    else if (System.IO.File.Exists(@"Sound\FX\" + f + ".wav"))
                        f += ".wav";
                    else
                        f = "Silence.wav";
                }
            } while (true);
        }

        /// <summary>
        /// Plays a note at a given freqency (Hz) for a given duration (ms) on the PC speaker
        /// </summary>
        private static void PCSpkr_Play_Note(int freqdur)
        {
            int freq, dur;
            freq = freqdur / (65536);
            dur = freqdur - (65536*freq);

            if (!mute)
            {
                if (freq <= 37 || freq >= 32767)
                    Thread.Sleep(dur);
                else
                    System.Console.Beep(freq, dur);
            }
            else
                Thread.Sleep(dur);
        }

        /// <summary>
        /// Loads a song from a file of frequency/duration pairs
        /// </summary>
        /// <param name="filepath"></param>
        /// <param name="notes"></param>
        private static void load(string filepath, bool music)
        {
            try 
            { 
                string[] lines = Graphics.Defaults.fileToStringArray(filepath);

                Queue<int> notes = new Queue<int>();
                int f, d;
                foreach (string s in lines)
                {
                    if (!s.Contains(@"//"))
                    {
                        f = int.Parse(s.Substring(0, s.IndexOf("\t")));
                        d = int.Parse(s.Substring(s.IndexOf("\t") + 1, s.Length - (s.IndexOf("\t") + 1)));
                        notes.Enqueue((f*65536) + d);
                    }
                }

                if (music)
                    pcspkr_Music = notes;
                else
                    pcspkr_FX = notes;
            }
            catch (Exception) { }
        }

        /// <summary>
        /// Fades the volume of music.
        /// </summary>
        public static void fade()
        {

        }
    }
}
